///////////////////Software Control Header///////////////////////////////////
// TString.cpp - Library to support string manipulation
//
// $LogThisFile$ \Projects\UtilCode\Lib\TString.cpp
// 
// V01.03 14/03/2001 2135
//  ToInteger(int Location) added.
// 
// V01.02 29/10/2000 2315
//  RemoveLeft() and operator+=(float) added.
// 
// V01.01 26/10/2000 2155
//  Various changes to deallocate memory only if exceeding m_Limit.
//  Default to 1024 bytes.
// 
// V01.00 11/10/2000 2141
//  Initial revision.
// 
// This code is freely distributed to help the development of complex
// software in any field using C++. The author is not held responsible
// for any damage caused by the usage of the code.
// The author should be acknowledged for the use of this piece of code in
// commercial software.
//
// Bug reports and suggestions are welcomed to report to
//
//          tzzstan@hotmail.com
/////////////////////////////////////////////////////////////////////////////
#include <string.h>

#include "TString.h"

/////////////////////////////////////////////////////////////////////////////
// CTString(char *string)
//
// Description: Construct from char*.
//
// string - Pointer to string kept in char array.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString::CTString(char *string, int LimitingSize)
{
   m_Limit = LimitingSize;
   m_Current = strlen(string) + 1;
   m_data = new char[m_Current];
   strcpy(m_data, string);
}

/////////////////////////////////////////////////////////////////////////////
// CTString(const CTString &string)
//
// Description: Copy construct from string.
//
// string - Reference to another CTString object.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString::CTString(const CTString &string)
{
   m_Limit = string.m_Limit;
   m_Current = string.m_Current;
   m_data = new char[m_Current];
   strcpy(m_data, string.m_data);
}

/////////////////////////////////////////////////////////////////////////////
// ~CTString()
//
// Description: Cleanup.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString::~CTString()
{
   delete []m_data;
}

/////////////////////////////////////////////////////////////////////////////
// GetChar(char Character, int FromLeftRight)
//
// Description: Find Character within string. Return its offset.
//
// Character - Character to search for.
// FromLeftRight - 0: Search from left.
//                 1: Search from right.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
int CTString::GetChar(char Character, int FromLeftRight)
{
   int Length = Size();
   int Position;

   if(FromLeftRight == 0)  // Search from left
   {
      for(Position = 0; Position < Length; Position++)
      {
         if(m_data[Position] == Character)
            return Position;
      }
   }
   else  // Search from right
   {
      for(Position = Length - 1; Position >= 0; Position--)
      {
         if(m_data[Position] == Character)
            return Position;
      }
   }
   return -1;  // Cannot find
}

/////////////////////////////////////////////////////////////////////////////
// InsertAt(int Location, char Character)
//
// Description: Insert Character at offset Location.
//
// Location - Offset to be inserted Character.
// Character - Character to insert.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
void CTString::InsertAt(int Location, char Character)
{
   int Length = strlen(m_data);
   char *t_ptr;

   if(Location > Length)   // Out of range index
      return;

   if(m_Current < Length + 2)
   {
      m_Current = Length + 2;
      t_ptr = new char[m_Current];
      strcpy(t_ptr, m_data);
      delete []m_data;
      m_data = t_ptr;
   }

   int CurIdx = Location;
   char tempChar = m_data[CurIdx];
   char tempChar2;
   m_data[CurIdx] = Character;
   while(tempChar)
   {
      tempChar2 = m_data[++CurIdx];
      m_data[CurIdx] = tempChar;
      tempChar = tempChar2;
   }
   m_data[++CurIdx] = tempChar;
}

/////////////////////////////////////////////////////////////////////////////
// InsertAt(int Location, CTString &string)
//
// Description: Insert string at offset Location.
//
// Location - Offset to insert string.
// string - string to be inserted.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
void CTString::InsertAt(int Location, CTString &string)
{
   int Length = string.Size();
   int Position;

   for(Position = Length - 1; Position >= 0; Position--)
      InsertAt(Location, string[Position]);
}

/////////////////////////////////////////////////////////////////////////////
// RemoveAt(int Location)
//
// Description: Remove one character from string.
//
// Location - Offset to the character to be removed.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
int CTString::RemoveAt(int Location)
{
   int Length = strlen(m_data);
   int Removed;
   char *t_ptr = m_data;
   
   if(Location < Length)
   {
      int Position = 0;
      char *L_ptr = t_ptr;
      while(m_data[Position])
      {
         *L_ptr = m_data[Position];
         if(Position != Location)
            L_ptr++;
         else
            Removed = *L_ptr;
         Position++;
      }
      *L_ptr = '\0';
   }
   else
      return 0;

   if(m_Current > m_Limit)
   {
      m_Current = Length;
      t_ptr = new char[m_Current];
      strcpy(t_ptr, m_data);
      delete []m_data;
      m_data = t_ptr;
   }

   return Removed;
}

/////////////////////////////////////////////////////////////////////////////
// RemoveLeft(int Characters)
//
// Description: Remove string from left of string
//
// Characters - Total number of characters to be removed.
//
// TSTan   28 Oct 2000
/////////////////////////////////////////////////////////////////////////////
void CTString::RemoveLeft(int Characters)
{
   int Idx;
   for(Idx = 0; Idx < Characters; Idx++)
      RemoveAt(0);
}

/////////////////////////////////////////////////////////////////////////////
// SetAt(int Location, char Character)
//
// Description: Set string at index Location to Character.
//
// Location - Offset from start of string to be replaced.
// Character - Character to replace to.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
void CTString::SetAt(int Location, char Character)
{
   if(Location < Size() && Location >= 0)
      m_data[Location] = Character;
}

/////////////////////////////////////////////////////////////////////////////
// Size()
//
// Description: Length of string without terminating NULL.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
int CTString::Size()
{
   return strlen(m_data);
}

/////////////////////////////////////////////////////////////////////////////
// ToInteger
//
// Description: Convert the string indexed by Location to integer
//
// TSTan   14 Mac 2001
/////////////////////////////////////////////////////////////////////////////
int CTString::ToInteger(int Location)
{
	if(Location >= Size() || Location < 0)
		return 0;

	int Value, Index, PosOrNeg;
	Value = 0;
	Index = Location;
	PosOrNeg = 1;
	
	if(m_data[Index] == '-')
	{
		Index++;
		PosOrNeg = -1;
	}

	do
	{
		if(m_data[Index] >= '0' && m_data[Index] <= '9')
		{
			Value *= 10;
			Value += m_data[Index] - '0';
		}
		else
			break;

		Index++;

	}while(1);

	Value *= PosOrNeg;

	return Value;
}	

/////////////////////////////////////////////////////////////////////////////
// ToUpper()
//
// Description: Change the whole string to upper case.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
void CTString::ToUpper()
{
   char *Ptr = m_data;
   while(*Ptr)
   {
      *Ptr = toupper(*Ptr);
      Ptr++;
   }
}

/////////////////////////////////////////////////////////////////////////////
// operator=(char *string)
//
// Description: Make this object containing string
//
// string - Pointer to string kept in char array.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString &CTString::operator=(char *string)
{
   int Length = strlen(string) + 1;

   if(m_Current < Length ||   // not enough memory
      ((m_Current > m_Limit) && (m_Current > Length))) // wasted memory
   {
      delete []m_data;
      m_Current = Length;
      m_data = new char[m_Current];
   }

   strcpy(m_data, string);
   return *this;
}

/////////////////////////////////////////////////////////////////////////////
// operator=(CTString &string)
//
// Description: Make this object image of string.
//
// string - Reference to source string.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString &CTString::operator=(CTString &string)
{
   m_Limit = string.m_Limit;
   *this = string.m_data;
   return *this;
}

/////////////////////////////////////////////////////////////////////////////
// operator==(char *string)
//
// Description: Check whether the string is the same with string pointed.
//
// string - Pointer to string kept in char array.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
bool CTString::operator==(char *string)
{
   return !strcmp(m_data, string);
}

/////////////////////////////////////////////////////////////////////////////
// operator==(CTString &string)
//
// Description: Check whether the string is the same with this object.
//
// string - Reference to string to be compared.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
bool CTString::operator==(CTString &string)
{
   return *this == string.m_data;
}

/////////////////////////////////////////////////////////////////////////////
// operator>(CTString &string)
//
// Description: True if this > string (alphabetically, length and lowercase
//              first).
//
// string - Reference to the second string.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
bool CTString::operator>(CTString &string)
{
   return strcmp(m_data, string.m_data) < 0;
}

/////////////////////////////////////////////////////////////////////////////
// operator<(CTString &string)
//
// Description: True if this < string (alphabetically, length and lowercase).
//
// string - Reference to the other string.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
bool CTString::operator<(CTString &string)
{
   return strcmp(m_data, string.m_data) > 0;
}

/////////////////////////////////////////////////////////////////////////////
// operator>=(CTString &string)
//
// Description: True if this >= string.
//
// string - Reference to the second string.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
bool CTString::operator>=(CTString &string)
{
   return (*this == string) || (*this > string);
}

/////////////////////////////////////////////////////////////////////////////
// operator<=(CTString &string)
//
// Description: True if this <= string.
//
// string - Reference to the second string.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
bool CTString::operator<=(CTString &string)
{
   return (*this == string) || (*this < string);
}

/////////////////////////////////////////////////////////////////////////////
// operator[](int index)
//
// Description: Return character offset from start of string.
//
// index - offset from start to get.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
char CTString::operator[](int index)
{
   return m_data[index];
}

/////////////////////////////////////////////////////////////////////////////
// operator+=(char character)
//
// Description: Append one character to the end of string.
//
// character - Character to be appended.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString &CTString::operator+=(char character)
{
   int NewLength = strlen(m_data) + 2;
   char *t_ptr;
   if(m_Current < NewLength)
   {
      m_Current = NewLength;
      t_ptr = new char[m_Current];
      strcpy(t_ptr, m_data);
      delete []m_data;
      m_data = t_ptr;
   }
   else
      t_ptr = m_data;

   while(*t_ptr) t_ptr++;  // Search to the end
   *t_ptr = character;
   t_ptr++;
   *t_ptr = '\0';

   return *this;
}

/////////////////////////////////////////////////////////////////////////////
// operator+=(char *string)
//
// Description: Concatenate string.
//
// string - String to be concatenated.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString &CTString::operator+=(char *string)
{
   int NewLength = strlen(m_data) + strlen(string) + 1;

   char *t_ptr;
   if(m_Current < NewLength)
   {
      m_Current = NewLength;
      t_ptr = new char[m_Current];
      strcpy(t_ptr, m_data);
      delete []m_data;
      m_data = t_ptr;
   }
   else
      t_ptr = m_data;

   strcat(t_ptr, string);

   return *this;
}

/////////////////////////////////////////////////////////////////////////////
// operator+=(CTString &string)
//
// Description: Append string.
//
// string - String to be appended.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString &CTString::operator+=(CTString &string)
{
   *this += string.m_data;
   return *this;
}

/////////////////////////////////////////////////////////////////////////////
// operator+=(int number)
//
// Description: Append integer(after converting to string) to the string.
//
// number - integer to be appended.
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString &CTString::operator+=(int number)
{
   char Buffer[80];
   ostrstream OutBuffer(Buffer, 80);
   OutBuffer << number << '\0';
   *this += OutBuffer.str();
   return *this;
}

/////////////////////////////////////////////////////////////////////////////
// operator+=(float number)
//
// Description: Append float(after converting to string) to the string.
//
// number - integer to be appended.
//
// TSTan   29 Oct 2000
/////////////////////////////////////////////////////////////////////////////
CTString &CTString::operator+=(float number)
{
   char Buffer[80];
   ostrstream OutBuffer(Buffer, 80);
   OutBuffer << setiosflags(ios::showpoint | ios::fixed)
      << setprecision(2) << number << '\0';
   *this += OutBuffer.str();
   return *this;
}

/////////////////////////////////////////////////////////////////////////////
// operator<<(ostream &os, CTString &string)
//
// Description: Support stream insertion operation.
//
// os - stream to be inserted.
// string - CTString to insert
//
// TSTan   11 Oct 2000
/////////////////////////////////////////////////////////////////////////////
ostream &operator<<(ostream &os, CTString &string)
{
   os << string.m_data;

   return os;
}



